/*
 * Infra Red protocols decoder
 * Author: Piotr Wjtowicz
 * 2016-2017
 * pw@elportal.pl
 */

#include "rc5dec.h"
#include "irdecoder.h"

#define MAXT 127

static void rc5AddBit(uint8_t bitValue);
// -----------------------------------------------
void rc5Postamble() {
    TROUT2 (
        RET1, RET0
    );    
    sm.productReady = 1;
    rc5AddBit(irsm.bitValueTmp);
}
// -----------------------------------------------
void rc5L1state() {
    TROUT3 (
        TR(US2TIME(RC5T), EPS, 1, rc5H1state),
        RET1, RET0
    );

    if (sm.productReady) {
        rc5AddBit(1);
        sm.productReady = 0;
    }
    irsm.bitValueTmp = 0;
}
// -----------------------------------------------
void rc5L2state() {
    TROUT4 (
        TR(US2TIME(RC5T), EPS, 1, rc5H2state),
        TR(US2TIME(2*RC5T), EPS, 1, rc5H1state),
        TR(MAXT, MAXT, 0, rc5Postamble),
        RET1
    );
    if (sm.productReady) {
        rc5AddBit(1);
    }
    sm.productReady = 1;
    irsm.bitValueTmp = 0;
}
// -----------------------------------------------
void rc5H1state() {
    TROUT4 (
        TR(US2TIME(RC5T), EPS, 0, rc5L1state),
        TR(US2TIME(2*RC5T), EPS, 0, rc5L2state),
        RET0, RET1
    );
    if (sm.productReady) {
        rc5AddBit(0);
    } else {
        sm.productReady = 1;
    }
}
// -----------------------------------------------
void rc5H2state() {
    TROUT3 (
        TR(US2TIME(RC5T), EPS, 0, rc5L2state),
        RET0, RET1
    );
    if (sm.productReady) {
        rc5AddBit(0);
        sm.productReady = 0;
    }
}
// -----------------------------------------------
static void rc5AddBit(uint8_t bitValue) {
    const uint8_t rc5smBytesLen[] = {1, 1, 5, 6};
    irsm.data[irsm.field] <<= 1;
    irsm.data[irsm.field] |= bitValue;

    if(irsm.bitCounter>=rc5smBytesLen[irsm.field]-1) {
        irsm.field++;
        irsm.bitCounter = 0;
    } else {
        irsm.bitCounter++;
    }

    if (irsm.field>3) {
        frame_ret.addr = irsm.data[2];
        frame_ret.cmd = irsm.data[3];
        frame_ret.pt = RC5_;
    }
}
